/*
 * Copyright (C) 2020 Mehmet Gökçay Kabataş - mgokcayk <mgokcaykdev@gmail.com>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

 // imports 
const St = imports.gi.St;
const Clutter = imports.gi.Clutter;
const Main = imports.ui.main;
const PanelMenu = imports.ui.panelMenu;
const PopupMenu = imports.ui.popupMenu;
const GObject = imports.gi.GObject;
const Lang = imports.lang;
const Mainloop = imports.mainloop;
const Slider = imports.ui.slider;
const GLib = imports.gi.GLib;
const Util = imports.misc.util;
const {Gtk, Gdk} = imports.gi;

const Me = imports.misc.extensionUtils.getCurrentExtension();
const ExtensionUtils = imports.misc.extensionUtils;

const Nbfc_Indicator = GObject.registerClass({
    GTypeName: 'NbfcIndicator',
	Name: 'Nbfc_Indicator',	// Class Name
}, class ServiceIndicator extends PanelMenu.Button {

	// Constructor
	_init() {
		super._init(0.0, 'Nbfc_Indicator', false);

		this.box = new St.BoxLayout(); // create box 
		
		// create icon
		let gicon = imports.gi.Gio.icon_new_for_string(Me.path + "/icons/nbfc-icon.png");
		let icon = new St.Icon({ gicon, icon_size: 16 });

		// create label for panel
		this.toplabel = new St.Label({ text: ' Nbfc ', 
			y_expand: true,
			y_align: Clutter.ActorAlign.CENTER });

		this.box.add_child(icon); // add icon to box
		this.box.add_child(this.toplabel); // add label to box

		this.actor.add_child(this.box); // add box to panel 

		this.settings = ExtensionUtils.getSettings(); // get settings from schemas

		// creatings menu items
		this.serviceEnable = new PopupMenu.PopupSwitchMenuItem('Service Enable ', {reactive: true});
		this.serviceEnable.setToggleState(0);
		this.readOnly = new PopupMenu.PopupMenuItem('Read Only ',{reactive:false});
		this.selectedConfigName = new PopupMenu.PopupMenuItem('Selected Config ', { reactive: false });
		this.temperature = new PopupMenu.PopupMenuItem('Temperature ' , { reactive: false });
		this.fanName = new PopupMenu.PopupMenuItem('Fan Display Name ', { reactive: false });
		this.autoControl = new PopupMenu.PopupSwitchMenuItem('Auto Control ', {reactive: true});
		this.autoControl.setToggleState(0);
		this.critMode = new PopupMenu.PopupMenuItem('Critical Mode ', {reactive: false});
		this.fanSpeed = new PopupMenu.PopupMenuItem('Fan Speed ', { reactive: false });
		this.tarFanSpeed = new PopupMenu.PopupMenuItem('Fan Target Speed', { reactive: false });
		this.slider = new Slider.Slider(0.5);
		this.sliderContainer = new PopupMenu.PopupBaseMenuItem( { reactive: false });
		this.sliderContainer.add(this.slider);
		this.settingsContainer = new PopupMenu.PopupMenuItem('Settings', {reactive:true});

		//this.debuggerG = new PopupMenu.PopupMenuItem('Debugger ', { reactive: false }); 
		
		// add menu items to menu
		//this.menu.addMenuItem(this.debuggerG);
		this.menu.addMenuItem(this.serviceEnable);
		this.menu.addMenuItem(this.readOnly);
		this.menu.addMenuItem(this.selectedConfigName);
		this.menu.addMenuItem(this.temperature);
		this.menu.addMenuItem(this.fanName);
		this.menu.addMenuItem(this.autoControl);
		this.menu.addMenuItem(this.critMode);
		this.menu.addMenuItem(this.fanSpeed);
		this.menu.addMenuItem(this.tarFanSpeed);
		this.menu.addMenuItem(this.sliderContainer);
		this.menu.addMenuItem(this.settingsContainer);


		this.location = this.settings.get_string('location');

		// handle switch and slider events with connect
		this.serviceEnable.connect('toggled', this._serviceEnableToggle.bind(this));

		this.autoControl.connect('toggled', this._autoControlToggle.bind(this));

	 	this.slider.connect('drag-end', this._onSliderValueChanged.bind(this));

		//handle signals
		this.settingsContainer.connect('activate', function () {
            Util.spawn(["gnome-shell-extension-prefs", Me.metadata.uuid]);
        });


	 	// create connection refuse counter
	 	this.connectionRefusedCounter = 0;

	 	// create update time previouse variable to check it is changed or not/
	 	this.updateTimePre = this.settings.get_int('update-time');

	 	//update display
	 	this._updateDisplay();

	 	// adding timer
	 	this._addTimer();		
 		}

		_serviceEnableToggle (cli, value)
		{
			if (value)
			{
				GLib.spawn_command_line_sync('mono '+ this.location +'/nbfc.exe start -e');
				this._updateDisplay();
			}else
			{
				GLib.spawn_command_line_sync('mono '+ this.location +'/nbfc.exe stop');
				this._updateDisplay();
			}
		}

		_autoControlToggle (cli, value)
		{
				if (value)
				{
					GLib.spawn_command_line_sync('mono '+ this.location +'/nbfc.exe set -a');
					this._updateDisplay();
				}else
				{
					let cmd = 'mono '+ this.location +'/nbfc.exe set -s' + String(this.fanSpeedValue);
					GLib.spawn_command_line_sync(cmd);
					this._updateDisplay();
				}
		}

		_updateDisplay()
		{	
		// check whether update timer changed from settings or not 
		if (this.updateTimePre != this.settings.get_int('update-time'))
		{
			this.updateTimePre = this.settings.get_int('update-time');
			this._updateTimeChanged();
		}

		this.location = this.settings.get_string('location');
		
		// run command line to get program output
		var commandLineOutput = GLib.spawn_command_line_sync('mono '+ this.location +'/nbfc.exe status -a').toString();
		var checker = commandLineOutput.split(",");
		var line = commandLineOutput.split("\n");

		// check whether connection refused or not from command line output
		if (checker[2].length < 20 && checker[2].length > 10)
		{
			var connectionRefused = new PopupMenu.PopupMenuItem("Connection Refused..", {reactive:false});
			
			var connectMessage1 = new PopupMenu.PopupMenuItem("Run : systemctl stop nbfc",{reactive:false});
			var connectMessage2 = new PopupMenu.PopupMenuItem("Run : systemctl enable nbfc --now",{reactive:false});
			
			// if connection refused add some menu items to warn user.
			if (this.connectionRefusedCounter < 1)
			{
				sep = new PopupMenu.PopupSeparatorMenuItem();
				this.menu.addMenuItem(sep);
				this.menu.addMenuItem(connectionRefused);
				this.menu.addMenuItem(connectMessage1);
				this.menu.addMenuItem(connectMessage2);
				fanSpeed.label.text = "Fan Speed : undefined";
				tarFanSpeed.label.text ="Fan Target Speed : undefined ";
				this.slider.value = 0.0;
				this.connectionRefusedCounter += 1;
				this._updateDisplay();
			}			
			return;
		}else
		{
			// if program become enable from connection refuse conditon, change some properties.
			if (this.connectionRefusedCounter == 1)
			{
				this.connectionRefusedCounter = 0;
				this._updateDisplay(); 
			}else // if program not in connection refused but there are some menu children, remove it.
			{
				let children = this.menu._getMenuItems();
				if (children.length > 11)
				{
					for(let i=children.length; i>11; i--)
					{
						children[children.length-1].destroy();
					}				
					this._updateDisplay();
				}			
			}
		}		
		
		// service status value and change switch w.r.t output
		var valServStat = line[0].split(": ")[1];
		if (valServStat == "True")
		{
			this.serviceEnable.setToggleState(1);
		}else
		{
			this.serviceEnable.setToggleState(0);
			this.slider.value = 0.0;
			this.toplabel.text = 'Nbfc - Service Disable';
		}

		/* READ ONLY PROPERTIES NOT WORK IN TERMINAL SO DISABLED. JUST ADDED TEXT
		var valReadOnly = line[1].split(": ")[1];
		
		readOnly.connect('toggle',Lang.bind(this, function(object, value)
		{	
			GLib.spawn_command_line_sync('mono '+ location +'/nbfc.exe stop');
			if (value) 
			{
				
				GLib.spawn_command_line_sync('mono '+ location +'/nbfc.exe start -r');
				this._updateDisplay();
			}else 
			{
				GLib.spawn_command_line_sync('mono '+ location +'/nbfc.exe start -e');
				this._updateDisplay();
			}
		}));
		if (valReadOnly == "True")
		{
			readOnly.setToggleState(1);
		}else
		{
			readOnly.setToggleState(0);
		}
		*/
		this.readOnly.label.text = 'Read Only : not work proper in terminal';

		// check config file names from setting page. if different apply config, else do nothing.
		var strCnfg = String(line[2].split(": ")[1]);
		var schemaCnfg = this.settings.get_string('configs').split(".")[0];
		if (strCnfg != schemaCnfg)
		{
			cmd = 'mono ' + this.settings.get_string('location') + "/nbfc.exe config --apply '" + schemaCnfg + "'";
			GLib.spawn_command_line_sync(cmd);
			var txtCnfg = schemaCnfg;
		}else 
		{
			var txtCnfg = strCnfg;
		}

		// edit menu items 
		var txtSelConFile = "Selected Config File : " + txtCnfg ;
		this.selectedConfigName.label.text = txtSelConFile;

		var txtTemp = "Temperature : " + String(line[3].split(": ")[1]);
		this.temperature.label.text = txtTemp;

		var txtFanName = "Fan Display Name : " + String(line[5].split(": ")[1]);
		this.fanName.label.text = txtFanName;

		var valAutoControl = line[6].split(": ")[1];
		if (valAutoControl == "True")
		{
			this.autoControl.setToggleState(1);
		}else
		{
			this.autoControl.setToggleState(0);
		}

		// if service is enable, get fan status.		
		if(line[7])
		{
			var valCritMode = line[7].split(": ")[1];
			if (valCritMode == "True")
			{
				this.critMode.label.text = 'Critic Mode : True. Cannot change by user.';
			}else
			{
				this.critMode.label.text = "Critic Mode : False. Cannot change by user.";
			}

			this.fanSpeedValue = line[8].split(": ")[1];
			var txtFanSpeed = "Fan Speed : " + String(this.fanSpeedValue) ;
			this.fanSpeed.label.text = txtFanSpeed;
			this.toplabel.text = "FS: "+txtFanSpeed.split(":")[1];
					
			var txtTarFanSpeed = "Fan Target Speed : " + String(line[9].split(": ")[1]) ;
			this.tarFanSpeed.label.text = txtTarFanSpeed;

			this.slider.value = this.fanSpeedValue / 100.0;
		}

	 	//debuggerG.label.text = String(slider.value);

	}

	// run command w.r.t slider value.		
	_onSliderValueChanged(cli, _slider) {
		if (this.autoControl.state == 0)
		{
			let cmd = 'mono '+ this.location +'/nbfc.exe set -s' + String(this.slider.value * 100);
			GLib.spawn_command_line_sync(cmd);
			this._updateDisplay();
		}
	}

	// update time when it is changed.
	_updateTimeChanged(){
        Mainloop.source_remove(this._timeoutId);
        this._addTimer();
    }

	// adding timer
	_addTimer(){
        this._timeoutId = Mainloop.timeout_add_seconds(this.settings.get_int('update-time'), Lang.bind(this, function (){
            this._updateDisplay();
            return true;
        }));
	}
	

	// destroy the class  
	destroy() {
		Mainloop.source_remove(this._timeoutId);
		this.parent();
	}
});


// Global variables for use as button to click 
let button;

// initialize function
function init() {}

// main function
function enable() {
	// create button
	button = new Nbfc_Indicator;

	// add button to panel
	Main.panel.addToStatusArea('PopupMenuExample', button, 0, 'right');
}

// destructor of main function
function disable() {
	button.destroy();	
}
